/*
 * The contents of this file are subject to the JOnAS Public License Version
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999,2001 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): Laurent Chauvirey, Frederic Maistre, Nicolas Tachker
 *
 * --------------------------------------------------------------------------
 * $Id: JConnectionFactory.java,v 1.1 2001/12/21 10:53:48 jonas Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_jms;

import java.io.Serializable;
import java.util.Iterator;
import java.util.LinkedList;
import javax.naming.NamingException;
import javax.naming.Reference;
import javax.naming.Referenceable;
import org.objectweb.jonas_jms.api.JmsManager;

public class JConnectionFactory implements Referenceable, Serializable {
    protected JmsManager jms;
    private String name;
    private LinkedList connectionpool = new LinkedList();

    /**
     * Constructor.
     */
    public JConnectionFactory(String name) {
	this.name = name;
	jms = JmsManagerImpl.getJmsManager();
    }

    // -----------------------------------------------------------------------
    // Internal Methods
    // -----------------------------------------------------------------------

    /**
     * Free a Connection and return it to the pool
     */
    public void freeJConnection(JConnection con) {
	TraceJms.debug( "JConnectionFactory: freeJConnection");
	synchronized(connectionpool) {
	    connectionpool.addLast(con);
	}
    }

    /**
     * Get a Connection from the pool for the specified user
     */
    public JConnection getJConnection(String user) {
	TraceJms.debug( "JConnectionFactory: getJConnection");
	JConnection con = null;
	synchronized(connectionpool) {
	    Iterator i = connectionpool.iterator();
	    while (i.hasNext()) {
		con = (JConnection) i.next();
		if (con.getUser().equals(user)) {
		    i.remove();
		    break;
		}
	    }
	}
	return con;
    }

    // -----------------------------------------------------------------------
    // Referenceable implementation
    // -----------------------------------------------------------------------

    public Reference getReference() throws NamingException {
	TraceJms.debug( "JConnectionFactory: getReference");
	return new Reference(getClass().getName(), "org.objectweb.jonas_jms.JObjectFactory", null);
    }
}
