/*
 * The contents of this file are subject to the JOnAS Public License Version
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999,2001 Bull S.A. All Rights Reserved.
 *
 * Contributor(s):
 *
 * --------------------------------------------------------------------------
 * $Id: JQueueConnectionFactory.java,v 1.1 2001/12/21 10:53:48 jonas Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_jms;

import javax.jms.JMSException;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.XAQueueConnectionFactory;

/**
 * JQueueConnectionFactory wraps a XAQueueConnectionFactory.
 */
public class JQueueConnectionFactory extends JConnectionFactory implements QueueConnectionFactory {
    
    private XAQueueConnectionFactory xaqcf;

    /**
     * Constructor.
     * The underlaying XAQueueConnectionFactory is found in the JmsManager.
     */
    public JQueueConnectionFactory(String name) {
	super(name);
	xaqcf = jms.getXAQueueConnectionFactory();
    }

    // -----------------------------------------------------------------------
    // QueueConnectionFactory implementation
    // -----------------------------------------------------------------------

    /**
     * Create a queue connection with default user identity.
     * @return a newly created queue connection.
     * @throws JMSException - if JMS Provider fails to create Queue Connection 
     * due to some internal error. required resources for a Queue Connection.
     * @throws JMSSecurityException - if client authentication fails due to 
     * invalid user name or password.
     */
    public QueueConnection createQueueConnection() throws JMSException {
	return createQueueConnection("jonas", "jonas");
    }

    /**
     * Create a queue connection with specified user identity. 
     * The connection is created in stopped mode. No messages will
     * be delivered until Connection.start method is explicitly called.
     * @param userName - the caller's user name
     * @param password - the caller's password
     * @throws JMSException - if JMS Provider fails to create Queue Connection 
     * due to some internal error. required resources for a Queue Connection.
     * @throws JMSSecurityException - if client authentication fails due to 
     * invalid user name or password.
     */
    public QueueConnection createQueueConnection(String userName, String password) throws JMSException {
	TraceJms.debug( "JQueueConnectionFactory: createQueueConnection");
	JQueueConnection qc = (JQueueConnection) getJConnection(userName);
	if (qc == null) {
	    qc = new JQueueConnection(this, xaqcf, userName, password);
	}
	return (QueueConnection) qc;
    }

} 
