/*
 * The contents of this file are subject to the JOnAS Public License Version
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations
 * under
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999,2001 Bull S.A. All Rights Reserved.
 *
 * Contributor(s):
 *
 * --------------------------------------------------------------------------
 * $Id: JQueueSession.java,v 1.1 2001/12/21 10:53:48 jonas Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_jms;

import javax.jms.JMSException;
import javax.jms.Queue;
import javax.jms.QueueBrowser;
import javax.jms.QueueReceiver;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.jms.TemporaryQueue;
import javax.jms.XAQueueConnection;
import javax.jms.XAQueueSession;
import javax.transaction.RollbackException;
import javax.transaction.SystemException;
import javax.transaction.Transaction;
import javax.transaction.TransactionManager;

public class JQueueSession extends JSession implements QueueSession {

    // Underlaying Objects
    protected XAQueueConnection xaqc;
    protected QueueSession qs = null;
    protected XAQueueSession xaqs = null;
    
    /**
     * Constructor
     */
    public JQueueSession(JConnection jconn, XAQueueConnection xaqc) {
	super(jconn);
	this.xaqc = xaqc;
    }

    // -----------------------------------------------------------------------
    // Internal Methods
    // -----------------------------------------------------------------------

    /**
     * Get the underlaying MOM Session.
     */
    protected Session getMOMSession() throws JMSException {
	return getMOMQueueSession();
    }

    protected QueueSession getMOMQueueSession() throws JMSException {
	Transaction tx = null;
	try {
	    tx = tm.getTransaction();
	} catch (SystemException e) {
	    TraceJms.error("JQueueSession getMOMSession: Cannot get Transaction");
	}
	if (tx == null) {
	    if (qs == null) {
		qs = xaqc.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
		jconn.sessionOpen(this);
	    }
	    return qs;
	} else {
	    if (xaqs == null) {
		xaqs = xaqc.createXAQueueSession();
		if (currtx != null) {
		    TraceJms.error("JQueueSession getMOMSession: mixed transactions");
		}
		currtx = tx;
		xares = xaqs.getXAResource();
		try {
		    tx.enlistResource(this);
		} catch (SystemException e) {
		    TraceJms.error("JQueueSession getMOMSession: cannot enlist session:"+e);
		    throw new JMSException(e.toString());
		} catch (RollbackException e) {
		    TraceJms.error("JQueueSession getMOMSession: transaction rolled back");
		    throw new JMSException(e.toString());
		}
	    } 
	    return xaqs.getQueueSession();
	}
    }

    protected void MOMSessionClose() {
	try {
	    if (xaqs != null) {
		xaqs.close();
		xaqs = null;
	    }
	    if (qs != null) {
		qs.close();
		qs = null;
		jconn.sessionClose(this);
	    }
	} catch (JMSException e) {
	    TraceJms.error("MOMSessionClose:"+e);
	}
    }

    // -----------------------------------------------------------------------
    // QueueSession Implementation
    // -----------------------------------------------------------------------

    /**
     *
     */
    public QueueBrowser createBrowser(Queue queue) throws JMSException {
	TraceJms.debug( "JQueueSession: createBrowser");
	return getMOMQueueSession().createBrowser(queue);
    }

    /**
     *
     */
    public QueueBrowser createBrowser(Queue queue, String messageSelector) throws JMSException {
	TraceJms.debug( "JQueueSession: createBrowser");
	return getMOMQueueSession().createBrowser(queue, messageSelector);
    }

    /**
     *
     */
    public Queue createQueue(String queueName) throws JMSException {
	TraceJms.debug( "JQueueSession: createQueue");
	return getMOMQueueSession().createQueue(queueName);
    }

    /**
     *
     */
    public QueueReceiver createReceiver(Queue queue) throws JMSException {
	TraceJms.debug( "JQueueSession: createReceiver");
	return getMOMQueueSession().createReceiver(queue);
    }

    /**
     *
     */
    public QueueReceiver createReceiver(Queue queue, String messageSelector) throws JMSException {
	TraceJms.debug( "JQueueSession: createReceiver");
	return getMOMQueueSession().createReceiver(queue, messageSelector);
    }
    
    /**
     *
     */
    public QueueSender createSender(Queue queue) throws JMSException {
	TraceJms.debug( "JQueueSession: createSender");
	return getMOMQueueSession().createSender(queue);
    }

    /**
     *
     */
    public TemporaryQueue createTemporaryQueue() throws JMSException {
	TraceJms.debug( "JQueueSession: createTemporaryQueue");
	return getMOMQueueSession().createTemporaryQueue();
    }


}
