/*
 * The contents of this file are subject to the JOnAS Public License Version
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations
 * under
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999,2001 Bull S.A. All Rights Reserved.
 *
 * Contributor(s):
 *
 * --------------------------------------------------------------------------
 * $Id: JTopicSession.java,v 1.1 2001/12/21 10:53:48 jonas Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_jms;

import javax.jms.InvalidDestinationException;
import javax.jms.JMSException;
import javax.jms.Session;
import javax.jms.TemporaryTopic;
import javax.jms.Topic;
import javax.jms.TopicPublisher;
import javax.jms.TopicSession;
import javax.jms.TopicSubscriber;
import javax.jms.XATopicConnection;
import javax.jms.XATopicSession;
import javax.transaction.RollbackException;
import javax.transaction.SystemException;
import javax.transaction.Transaction;
import javax.transaction.TransactionManager;

public class JTopicSession extends JSession implements TopicSession {

    // Underlaying Objects
    protected XATopicConnection xatc;
    protected TopicSession ts = null;
    protected XATopicSession xats = null;

    /**
     * Constructor
     */
    public JTopicSession(JConnection jconn, XATopicConnection xatc) {
	super(jconn);
	this.xatc = xatc;
    }

    // -----------------------------------------------------------------------
    // Internal Methods
    // -----------------------------------------------------------------------

    /**
     * Get the underlaying MOM Session.
     */
    protected Session getMOMSession() throws JMSException {
	return getMOMTopicSession();
    }

    protected TopicSession getMOMTopicSession() throws JMSException {
	Transaction tx = null;
	try {
	    tx = tm.getTransaction();
	} catch (SystemException e) {
	    TraceJms.error("JTopicSession getMOMSession: Cannot get Transaction");
	}
	if (tx == null) {
	    if (ts == null) {
		ts = xatc.createTopicSession(false, Session.AUTO_ACKNOWLEDGE);
		jconn.sessionOpen(this);
	    }
	    return ts;
	} else {
	    if (xats == null) {
		xats = xatc.createXATopicSession();
		if (currtx != null) {
		    TraceJms.error("JTopicSession getMOMSession: mixed transactions");
		}
		currtx = tx;
		xares = xats.getXAResource();
		try {
		    tx.enlistResource(this);
		} catch (SystemException e) {
		    TraceJms.error("JTopicSession getMOMSession: cannot enlist session:"+e);
		    throw new JMSException(e.toString());
		} catch (RollbackException e) {
		    TraceJms.debug("JTopicSession getMOMSession: transaction rolled back");
		    throw new JMSException(e.toString());
		}
	    } 
	    return xats.getTopicSession();
	}
    }

    protected void MOMSessionClose() {
	try {
	    if (xats != null) {
		xats.close();
		xats = null;
	    }
	    if (ts != null) {
		ts.close();
		ts = null;
		jconn.sessionClose(this);
	    }
	} catch (JMSException e) {
	    TraceJms.error("MOMSessionClose:"+e);
	}
    }

    // -----------------------------------------------------------------------
    // TopicSession Implementation
    // -----------------------------------------------------------------------

    /**
     *
     */
    public Topic createTopic(String topicName) throws JMSException {
	TraceJms.debug( "JTopicSession: createTopic");
	return getMOMTopicSession().createTopic(topicName);
    }

    /**
     *
     */
    public TopicSubscriber createSubscriber(Topic topic) 
	throws JMSException{
	TraceJms.debug( "JTopicSession: createSubscriber");
	return getMOMTopicSession().createSubscriber(topic);
    }
    
    /**
     *
     */
    public TopicSubscriber createSubscriber(Topic topic, 
					    String messageSelector, 
					    boolean noLocal) throws JMSException {
	TraceJms.debug( "JTopicSession: createSubscriber");
	return getMOMTopicSession().createSubscriber(topic, messageSelector, noLocal);
    }

    /**
     *
     */
    public TopicSubscriber createDurableSubscriber(Topic topic, 
						   String name) throws JMSException {
	TraceJms.debug( "JTopicSession: createDurableSubscriber");
	return getMOMTopicSession().createDurableSubscriber(topic, name);
    }
 
    /**
     *
     */
    public TopicSubscriber createDurableSubscriber(Topic topic, 
						   String name, 
						   String messageSelector, 
						   boolean noLocal) throws JMSException{ 
	TraceJms.debug( "JTopicSession: createDurableSubscriber");
	return getMOMTopicSession().createDurableSubscriber(topic, name, messageSelector, noLocal);
    }
   
    /**
     *
     */
    public TopicPublisher createPublisher(Topic topic) throws JMSException {
	TraceJms.debug( "JTopicSession: createPublisher");
	return getMOMTopicSession().createPublisher(topic);
    }

    /**
     *
     */
    public TemporaryTopic createTemporaryTopic() throws JMSException {
	TraceJms.debug( "JTopicSession: createTemporaryTopic");
	return getMOMTopicSession().createTemporaryTopic();
    }
  
    /**
     *
     */
    public void unsubscribe(java.lang.String name) throws JMSException, InvalidDestinationException {
	TraceJms.debug( "JTopicSession: unsubscribe");
	getMOMTopicSession().unsubscribe(name);
    }
}
