/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 2001 Bull S.A. All Rights Reserved.
 *
 * Contributor(s):
 *
 * --------------------------------------------------------------------------
 * $Id: JmsJmxManagement.java,v 1.2 2001/12/21 10:53:48 jonas Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_jms;

import java.util.Enumeration;
import javax.jms.Queue;
import javax.jms.QueueConnectionFactory;
import javax.jms.XAQueueConnectionFactory;
import javax.jms.Topic;
import javax.jms.TopicConnectionFactory;
import javax.jms.XATopicConnectionFactory;
import javax.transaction.TransactionManager;

/**
 * JMS Administration  interface.
 * must be implemented for each jms provider
 */
public interface JmsJmxManagement {
    /**
     * Management method
     * @return the current number of Topic Jms Connection Factory
     */ 
    public int getCurrentNumberOfJmsTopicConnectionFactory();

    /**
     * Management method
     * @return the current number of Queue Jms Connection Factory
     */ 
    public int getCurrentNumberOfJmsQueueConnectionFactory();

    /**
     * Management method
     * @return the current number of Topic Jms Destination
     */ 
    public int getCurrentNumberOfJmsTopicDestination();

    /**
     * Management method
     * @return the current number of Queue Jms Destination
     */ 
    public int getCurrentNumberOfJmsQueueDestination();

    /**
     * Management method
     * Remove a Jms destination
     * @param String jndi name
     */
    public void removeJmsDestination(String jndiName) throws Exception;

    /**
     * Management method
     * @return String name of default Queue Connection factory
     */
    public String getDefaultQueueConnectionFactoryName();


    /**
     * Management method
     * @return String name of default Topic Connection factory
     */
    public String getDefaultTopicConnectionFactoryName();
}

